using MeshCore: ShapeColl, IncRel
using MeshCore: shapedesc, nshapes, Attrib
import Base.show

struct Mesh
    increls::Dict{String, IncRel} # dictionary of incidence relations
    name::String # name of the mesh
end

"""
    Mesh()

Define the mesh with default name and empty dictionary of incidence relations.
"""
function Mesh()
    # Mesh(Dict{String, ShapeColl}(), Dict{String, IncRel}())
    Mesh(Dict{String, IncRel}(), "mesh")
end

"""
    Mesh(s::String)

Define the mesh with an empty dictionary of incidence relations.
"""
function Mesh(s::String)
    # Mesh(Dict{String, ShapeColl}(), Dict{String, IncRel}())
    Mesh(Dict{String, IncRel}(), s)
end
 
"""
    increl(m::Mesh, s::String)

Retrieve the named incidence relation.
"""
increl(m::Mesh, s::String) = m.increls[s]

"""
    insert!(m::Mesh, s::String, increl::IncRel)

Insert the incidence relation under the name `s`.
"""
insert!(m::Mesh, s::String, increl::IncRel) = (m.increls[s] = increl)

"""
    insert!(m::Mesh, increl::IncRel)

Insert the incidence relation under its own name.
"""
insert!(m::Mesh, increl::IncRel) = (m.increls[increl.name] = increl)

"""
    Base.summary(a::T) where {T<:Attrib}

Form a brief summary of an attribute.
"""
function Base.summary(a::T) where {T<:Attrib}
    return "$(a.name) = Attrib $(typeof(a.val))"
end

"""
    Base.summary(sc::S) where {S<:ShapeColl}

Form a brief summary of the shape collection.
"""
function Base.summary(sc::S) where {S<:ShapeColl}
    s = "$(sc.name) = ShapeColl{$(shapedesc(sc).name)}, $(nshapes(sc)) shapes"
    for a in values(sc.attributes)
        s = s * ", " * summary(a)
    end
    return s
end

"""
    Base.summary(ir::IncRel)

Form a brief summary of the incidence relation.
"""
function Base.summary(ir::IncRel)
    return "$(ir.name) = IncRel " * summary(ir.left) * " -> " * summary(ir.right)
end

"""
    Base.summary(m::Mesh)

Form a brief summary of the mesh.
"""
function Base.summary(m::Mesh)
    s = "Mesh $(m.name): "
    for ir in m.increls
        s = s * "increl[\"$(ir[1])\"] = " * summary(ir[2])
    end
    return s
end
